<?php
/****************************************************
  Coppermine 1.5.x Plugin - Facebook OAuth (fb_oauth)
  ***************************************************
  Copyright (c) 2010 Jon Troncoso, 2011 Roland Fulde
  **************************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

// We may or may not have this data based on a $_GET or $_COOKIE based session.
//
// If we get a session here, it means we found a correctly signed session using
// the Application Secret only Facebook and the Application know. We dont know
// if it is still valid until we make an API call using the session. A session
// can become invalid if it has already expired (should not be getting the
// session back in this case) or if the user logged out of Facebook.

$thisplugin->add_action('plugin_install','fb_oauth_install');
$thisplugin->add_action('plugin_uninstall','fb_oauth_uninstall');
$thisplugin->add_action('page_start','fb_oauth_init');
$thisplugin->add_filter('sys_menu','fb_oauth_add_sysmenu_button');

// Facebook says the fb-root and the Facebook Javascript code should be
// put as much down at the bottom (HTML-wise) of the page as possible,
// to not slow down the actual page.
$thisplugin->add_filter('gallery_footer','fb_oauth_putHtml');

// just for the case that CPG option comment_promote_registration is set
// There is a login request text that should be adapted to not only promote
// normal login but also Facebook login.
$thisplugin->add_filter('theme_add_comment','fb_oauth_customize_login_requested_text');

// for showing who of the user's Facebook friends can see an album
$thisplugin->add_action('theme_thumbnails_wrapper_end','fb_oauth_whocanseethisalbum');
    

// Install the plugin
function fb_oauth_install() {
	global $CONFIG, $thisplugin;

          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_appid', '0')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_secret', '00000000000000000000000000000000')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_defaultgroup', '3')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_autologin', '0')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_want_bigloginbutton', '0')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_bigloginbutton_pos', 'style=\"position: absolute; top: 2px; right:2px; float:right\"')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_addbuttonforce', '0')");
          cpg_db_query("INSERT IGNORE INTO {$CONFIG['TABLE_CONFIG']} (`name`, `value`) VALUES ('plugin_fb_oauth_whocanseethisalbum', '0')");

	// about the table
	// -- be generous about the Facebook uid, you never know how logn Facebook might make it
	// -- the FOREIGN KEY REFERENCES CASCADE clauses seem to be ignored completely; in mysql docs it sounds as if that only works with InnoDB?
	$query = <<< EOT
		CREATE TABLE IF NOT EXISTS {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap (
			cpg_user_id int(11) NOT NULL,
		  	fb_uid varchar(255) NOT NULL UNIQUE,
		  	CONSTRAINT FOREIGN KEY (cpg_user_id)
		  	     REFERENCES {$CONFIG['TABLE_USERS']} (user_id)
		  	     ON DELETE CASCADE ON UPDATE CASCADE
  	  	) COMMENT='maps Facebook uid to Coppermine user_id';
EOT;
	cpg_db_query($query);	
	return true;
}

// Uninstall the plugin
function fb_oauth_uninstall() {
	global $CONFIG, $thisplugin;

	$query = <<< EOT
		DROP TABLE `{$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap`
EOT;
	cpg_db_query($query);	
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_appid'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_secret'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_defaultgroup'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_autologin'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_want_bigloginbutton'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_bigloginbutton_pos'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_addbuttonforce'");
          cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE name = 'plugin_fb_oauth_whocanseethisalbum'");
	return true;
}

function fb_oauth_customize_login_requested_text($html)
{
    require("./plugins/fb_oauth/fb_init.php");
    global $lang_display_comments;
    if(!$fb_want_bigloginbutton) {
        // Okay, now this is tricky: Whereever there is anything that looks like a % formatststing, escape it as %%,
        // because this will be passed through sprintf by Coppermine.
        $printloginurl = preg_replace('/%/','%%', htmlentities($facebook->getLoginUrl($login_params)));
        $printimgurl = preg_replace('/%/','%%', $fb_img_html);
    }
    $lang_display_comments['log_in_to_comment'] = preg_replace(
            array('/%FBLINKSTART%/','/%FBLINKEND%/'),
            $fb_want_bigloginbutton ? // Let's distinguish here -- I think the fb:login-button is uglier
                                      // but it uses a nicer popup window without having to use custom
                                      // Javascript
                array('<fb:login-button size="small">','</fb:login-button>') :
                array('<a href="'.$printloginurl.'">'.$printimgurl,'</a>'),
            $lang_plugin_fb_oauth['log_in_to_comment']);
    return $html;
}

function fb_oauth_putHtml($html) {
    require("./plugins/fb_oauth/fb_init.php");
    global $USER_DATA, $CONFIG;
    $fbautologincode = '';
    $fbloginbutton = '';
    $appId = $CONFIG['plugin_fb_oauth_appid'];
    // Facebook autologin only if no user is logged on. Could be
    // counterproductive otherwise.
    // (CPG may still have non-Facebook users!)
    if($USER_DATA['user_id']==0) {
        if($fb_autologin) {
            $fbautologincode = <<< EOT
    FB.Event.subscribe('auth.login', function(response) {
        window.location.reload();
    });
EOT;
/* // This Javascript piece sounds also good, but I think this would be double effort and therefore too much.
    FB.getLoginStatus(function(response) {
        if (response.session) {
            window.location.reload();
        }
    });
*/
        }
        if ($fb_want_bigloginbutton) {
            $fbloginbutton = '<span '.$fb_bigloginbutton_positioncode.'><fb:login-button>'.$lang_plugin_fb_oauth['login_lnk'].'</fb:login-button></span>';
        }
    }
    $facebookHtml = <<< EOT
<div id="fb-root">
<script type="text/javascript" src="http://connect.facebook.net/{$lang_plugin_fb_oauth['fb_langcode']}/all.js"></script>
<script type="text/javascript">
    FB.init({
      appId   : '{$appId}',
      status  : true,
      cookie  : true,
      xfbml   : true
    });
$fbautologincode
</script>
$fbloginbutton
</div>

EOT;
  return $facebookHtml.$html;
}

function fb_oauth_add_sysmenu_button($menu) {
	require("./plugins/fb_oauth/fb_init.php");
	global $template_sys_menu_spacer, $USER_DATA;
	
	if($fb_add_sysmenu_button_brute_force) {
	    return $menu;
	}

	$me = null;
	// Session based API call.
	if (!empty($session)) {
	  try {
	    $uid = $facebook->getUser();
	    $me = $facebook->api('/me');
	  } catch (FacebookApiException $e) {
	    error_log($e);
	  }
	}

	// add buttons only when the user is not logged on or logged on via facebook
	if (!$USER_DATA['user_id'] || $me) {
	    // login or logout url will be needed depending on current user state.
	    if ($me) { // logout button, do not add, just change the existing one
                $logoutButton = fb_oauth_find_button($menu, '{LOGOUT_LNK}');
                $buttonLnk = htmlspecialchars($facebook->getLogoutURL());
		$buttonTtl = '<img src="https://graph.facebook.com/'.$uid.'/picture" border="0" height="10" style="height:1em;" alt="" class="icon" />';
                              // style= with "em" sizing is relative to font size; the old height attribute actually not needed anymore.
		$buttonHelp = $lang_plugin_fb_oauth['logout_title'];
                $menu[$logoutButton][0] = $buttonTtl.$menu[$logoutButton][0];
		$menu[$logoutButton][1] = $buttonHelp;
		$menu[$logoutButton][2] = $buttonLnk;
            } else { // login button, add one and modify the existing one to avoid confusion
                $loginButton = fb_oauth_find_button($menu, '{LOGIN_LNK}');
	        $buttonLnk = htmlspecialchars($facebook->getLoginUrl($login_params));
		$buttonTtl = $fb_img_html;
		$buttonTtl .= $lang_plugin_fb_oauth['login_lnk'];
		$buttonHelp = $lang_plugin_fb_oauth['login_title'];
		$menu[$loginButton][0] = $lang_plugin_fb_oauth['normal_login_lnk'];

		$new_button = array();
		$new_button[0][0] = $buttonTtl;
		$new_button[0][1] = $buttonHelp;
		$new_button[0][2] = $buttonLnk;
		$new_button[0][3] = 'facebooklogin';
		$new_button[0][4] = $template_sys_menu_spacer;
		$new_button[0][5] = 'rel="nofollow"';

		array_splice($menu, $loginButton, 0, $new_button);		
            }
  	}
	return $menu;
}

function fb_oauth_find_button($menu, $whichOne, $byWhat=0) {
    foreach($menu as $bkey => $button) {
        if ($button[$byWhat] == $whichOne) {
            return $bkey;
        }
    }
}

function fb_oauth_add_sysmenu_button_brute_force() {
	require("./plugins/fb_oauth/fb_init.php");
	global $template_sys_menu, $USER_DATA;
	
	$me = null;
	// Session based API call.
	if (!empty($session)) {
	  try {
	    $uid = $facebook->getUser();
	    $me = $facebook->api('/me');
	  } catch (FacebookApiException $e) {
	    error_log($e);
	  }
	}

	// add buttons only when the user is not logged on or logged on via facebook
	if (!$USER_DATA['user_id'] || $me) {
	    // login or logout url will be needed depending on current user state.
	    if ($me) { // logout button, do not add, just change the existing one
                      $placeholders = array('{LOGOUT_TGT}', '{LOGOUT_ICO}', '{LOGOUT_TITLE}');
                      $replacements = array(
                                          htmlspecialchars($facebook->getLogoutUrl()),
                                          '<img src="https://graph.facebook.com/'.$uid.'/picture" border="0" height="10" style="height:1em;" alt="" class="icon" />',
                                          $lang_plugin_fb_oauth['logout_title']
                                        );
                      $template_sys_menu = str_replace($placeholders, $replacements, $template_sys_menu);
            } else { // login button, add one and modify the existing one to avoid confusion
                      $startmarker = '<!-- BEGIN login -->';
                      $endmarker = '<!-- END login -->';
                      $startpos = strpos($template_sys_menu, $startmarker);
                      $endpos = strpos($template_sys_menu, $endmarker);
                      $loginpart = substr($template_sys_menu, $startpos, $endpos-$startpos+strlen($endmarker));
                      $placeholders = array('{LOGIN_TGT}', '{LOGIN_ICO}', '{LOGIN_LNK}', '{LOGIN_TITLE}', $startmarker, $endmarker);
                      $replacements = array(
                                          htmlspecialchars($facebook->getLoginUrl($login_params)),
                                          $fb_img_html,
                                          $lang_plugin_fb_oauth['login_lnk'],
                                          $lang_plugin_fb_oauth['login_title'],
                                          '<!-- BEGIN facebooklogin -->',
                                          '<!-- END facebooklogin -->'
                                        );
                      $loginpart = str_replace($placeholders, $replacements, $loginpart); // this sets the texts for the FB button
                      $template_sys_menu = str_replace('{LOGIN_LNK}', $lang_plugin_fb_oauth['normal_login_lnk'], $template_sys_menu); // this changes the text for the existing button
                      $template_sys_menu = str_replace($startmarker, $loginpart.$startmarker, $template_sys_menu); // this finally adds the facebook button
              }
      }
}

function fb_oauth_init() {
	require("./plugins/fb_oauth/fb_init.php");
	global $CONFIG, $USER_DATA, $cpg_udb, $PLUGIN_FB_OAUTH_UID;

	if($fb_add_sysmenu_button_brute_force) {
	    fb_oauth_add_sysmenu_button_brute_force();
	}
	if($USER_DATA['user_id']!=0 && empty($session)) {
            return false;
        }
	$me = null;
	// Session based API call.
	if (!empty($session)) {
	  try {
	    $uid = $facebook->getUser();
	    $PLUGIN_FB_OAUTH_UID = $uid;	// for provisioning other customizations, e.g. anycontent.php
	    $uid = mysql_real_escape_string ($uid);	// We will use it in SQL String here (and only in those!)
	    $me = $facebook->api('/me');
	  } catch (FacebookApiException $e) {
	    error_log($e);
	  }
		//Log out if you detect a facebook logout
		if($USER_DATA['user_id']!=0){

			if(!$me){
				$cpg_udb->logout();
				// After logout stay on same page, too.
				// Is that useful?
				// Well, as long as we have the logout msg I think yes. Without it, it might be confusing for the end user to stay on a page where he might not have any logged-out privileges left.
				cpgRedirectPage(fb_oauth_getScriptNameParams(), $lang_plugin_fb_oauth['after_fb_logout_msg'], $lang_plugin_fb_oauth['after_fb_logout_msg'], 3);
			}
			return false;
		}

		if($me){
			$needtocreateuser = true;
			$sql = "SELECT cpg_user_id FROM {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap WHERE fb_uid='$uid'";
			$result = cpg_db_query($sql);
			if(mysql_num_rows($result) > 0) {
				$row = mysql_fetch_array($result);
			        $cpguserid = $row['cpg_user_id'];
				mysql_free_result($result);
				
				$sql = "SELECT user_id FROM {$CONFIG['TABLE_USERS']} WHERE user_id = $cpguserid";
				$result = cpg_db_query($sql);
				if(mysql_num_rows($result) > 0) {
				      // we have the mapping and we have the user.
				      $needtocreateuser = false;
				} else {
				      // dammit, we have the user in the mapping table but not in Coppermine.
				      // Maybe he has been deleted earlier but the mapping did not go away
				      // (see comment above about MySQL's FOREIGN KEY ... CASCADE clause).
				      // I suppose it is best/cleanest to then also delete the old mapping entry
				      // and start over like with anyone else
				      $sql = "DELETE FROM {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap WHERE fb_uid = '$uid'";
				      $result = cpg_db_query($sql);
				      $needtocreateuser = true;
				}
			}
			if ($needtocreateuser) {
			        $user_language = fb_oauth_select_language($me['locale']);
				$attempt=-1;
				while(1) {
				        // Let's generate a Coppermine username from the Facebook username in a Really Good(tm) way.
				        // Want to cope also with possible multiple different people all named John Doe, who all do not share their Birthdate from Facebook,
				        // So first trying without birthdate, then with, and if that still does not work, find a number to append.
				        // Note well that due to the mapping table, a Coppermine administrator can change the Coppermine username for this user without any problem.
					switch($attempt) {
						case -1:
							$usern = $me['first_name'].$me['last_name'];
							break;
						case 0:
							$usern = $me['first_name'].$me['last_name'].preg_replace('/\//', '', $me['birthday']);
							break;
						default:
							$usern = $me['first_name'].$me['last_name'].$attempt;
							break;
					}
					$sql = "SELECT user_id FROM {$CONFIG['TABLE_USERS']} WHERE user_name = '{$usern}'";
					$result = cpg_db_query($sql);
					if(mysql_num_rows($result) == 0) {
						break;
					}
					$attempt++;
				}

				// Random password -- yeah, we want that for security
				// If the user who originally came from Facebook wants to change it
				// in his Coppermine profile, he is free to do so.
				$password = fb_oauth_genRandomString(40);
			        $encpassword = md5($password);

				$sql = "INSERT INTO {$CONFIG['TABLE_USERS']} (user_regdate, user_active, user_actkey, user_name, user_password, user_group, user_email, user_profile1, user_profile2, user_profile3, user_profile4, user_profile5, user_profile6, user_language) VALUES (NOW(), 'YES', '', '$usern', '$encpassword', '$fb_defaultgroup', '{$me['email']}', '', '', '', '', '', '', '$user_language')";
				$result = cpg_db_query($sql);

				$cpguserid = mysql_insert_id();
				$sql="INSERT INTO {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap (cpg_user_id, fb_uid) values ($cpguserid, '$uid')";
				$result = cpg_db_query($sql);
			}
			$USER_DATA = fb_oauth_direct_login ($cpguserid);
			if($USER_DATA['user_id']) {
				cpgRedirectPage(fb_oauth_getScriptNameParams(), $lang_plugin_fb_oauth['after_fb_login_msg'], $lang_plugin_fb_oauth['after_fb_login_msg'], 3);
			} else {
				$cpg_udb->logout();
				// Do NOT redirect here, or you'll get into an infinite loop.
				// This is a legitimate state! E.g. if a Facebook-coming user has been set to
				// inactive in Coppermine.
                                msg_box('', $lang_plugin_fb_oauth['fb_login_fail'], '', '', 'error');
			}
		}
	}
	
}

function fb_oauth_getScriptNameParams()
{
    // Stay on same page -- it would have been nice if I could have used simply
    // cpgGetScriptNameParams('session')
    // for that -- but apparently not only cpgGetScriptNameParams but also Inspekt
    // are confused by that parameter.
    // oh right, and I do not want &amp; (which cpgGetScriptNameParams() does)
    // but literal & in the URL for redirect.
    $superCage = Inspekt::makeSuperCage();
    $match = $superCage->server->getRaw('SCRIPT_NAME'); // We'll sanitize the script path later
    $filename = ltrim(strrchr($match, '/'), '/'); // Drop everything untill (and including) the last slash, this results in the file name only
    if (!preg_match('/^(([a-zA-Z0-9_\-]){1,})((\.){1,1})(([a-zA-Z]){2,6})+$/', $filename)) { // the naming pattern we check against: an infinite number of lower and upper case alphanumerals plus allowed special chars dash and underscore, then one (and only one!) dot, then between two and 6 alphanumerals in lower or upper case
        $filename = 'index.php'; // If this doesn't match, default to the index page
    }
    $match = $superCage->server->getRaw('QUERY_STRING');
    // We can thow away that session parameter now,
    // assuming(!) that it is the last one(!!)
    // as we already got all Facebook information
    // (and I suppose the Facebook API has the cookie set at this point)
    $match = preg_replace('/&session=.*$/', '', $match);
    // so and now just look at the rest, and whether it is considerably sane
    if (!preg_match('/^[a-zA-Z0-9&=_\/.]+$/', $match)) {
        $match = '';
    }
    $currentPage = $filename.'?'.$match;
    return $currentPage;
}

function fb_oauth_genRandomString($length) {
	$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$string = '';
	for ($i = 0; $i < $length; $i++) {
		$string .= $characters[mt_rand(0, strlen($characters))];
	}
	return $string;
}


function fb_oauth_select_language($code) {
    global $USER;
    $code = strtolower($code);
    switch (substr($code, 0, 4)) {
        case 'en-gb':
        case 'en_gb': return 'english_gb';
        case 'pt-br':
        case 'pt_br': return 'brazilian_portuguese';
        case 'zh-tw':
        case 'zh_tw': return 'chinese_big5';
        default:
            switch (substr($code, 0, 2)) {
                case 'af': return 'afrikaans';
                case 'ar': return 'arabic';
                case 'be': return 'belarusian';
                case 'bg': return 'bulgarian';
                case 'ca': return 'catalan';
                case 'cs': return 'czech';
                case 'da': return 'danish';
                case 'de': return 'german';
                case 'el': return 'greek';
                case 'en': return 'english';
                case 'es': return 'spanish';
                case 'et': return 'estonian';
                case 'fa': return 'farsi';
                case 'fi': return 'finnish';
                case 'fr': return 'french';
                case 'gl': return 'galician';
                case 'he': return 'hebrew';
                case 'hi': return 'hindi';
                case 'hr': return 'croatian';
                case 'hu': return 'hungarian';
                case 'id': return 'indonesian';
                case 'is': return 'icelandic';
                case 'it': return 'italian';
                case 'ja': return 'japanese';
                case 'ko': return 'korean';
                case 'ka': return 'georgian';
                case 'kk': return 'kazakh';
                case 'ls': return 'slovenian';
                case 'lt': return 'lithuanian';
                case 'lu': return 'luxembourgish';
                case 'lv': return 'latvian';
                case 'ms': return 'malay';
                case 'nl': return 'dutch';
                case 'no': return 'norwegian';
                case 'pl': return 'polish';
                case 'pt': return 'portuguese';
                case 'ro': return 'romanian';
                case 'ru': return 'russian';
                case 'sk': return 'slovak';
                case 'sl': return 'slovenian';
                case 'sq': return 'albanian';
                case 'sr': return 'serbian';
                case 'sv': return 'swedish';
                case 'th': return 'thai';
                case 'tr': return 'turkish';
                case 'uk': return 'ukrainian';
                case 'zh': return 'chinese_gb';
                default: return $USER['lang'];
            }
    }
}

// imitate what bridge/coppermine.inc.php does for login()
function fb_oauth_direct_login($cpguserid) {
                    global $CONFIG, $cpg_udb;

                    // Create the session_id from concat(cookievalue,client_id)
                    $session_id = $cpg_udb->session_id.$cpg_udb->client_id;

                    // Check for user in users table
                    $sql =  "SELECT user_id, user_name, user_password FROM {$CONFIG['TABLE_USERS']} WHERE user_id = $cpguserid AND user_active = 'YES'";
                    $results = cpg_db_query($sql);

                    // If exists update lastvisit value, session, and login
                    if (mysql_num_rows($results)) {

                            // Update lastvisit value
                            $sql =  "UPDATE {$CONFIG['TABLE_USERS']} SET user_lastvisit = NOW() WHERE user_id = $cpguserid AND user_active = 'YES'";
                            cpg_db_query($sql, $cpg_udb->link_id);

                            $USER_DATA = mysql_fetch_assoc($results);
                            mysql_free_result($results);

                            setcookie( $cpg_udb->client_id, $cpg_udb->session_id, 0, $CONFIG['cookie_path'] );

                            // Update guest session with user's information
                            $sql  = "UPDATE {$CONFIG['TABLE_PREFIX']}sessions SET ";
                            $sql .= "user_id = {$USER_DATA['user_id']} ";
                            $sql .= "WHERE session_id = '" . md5($session_id) . "'";
                            cpg_db_query($sql, $cpg_udb->link_id);

                            return $USER_DATA;
                    } else {

                            return false;
                    }
            }



function fb_oauth_whocanseethisalbum() {
        global $CONFIG;
        if (!$CONFIG['plugin_fb_oauth_whocanseethisalbum']) {
                return;
        }

	require("./plugins/fb_oauth/fb_init.php");
        $fbcookie = fb_oauth_get_facebook_cookie($CONFIG['plugin_fb_oauth_appid'], $CONFIG['plugin_fb_oauth_secret']);
        if (!$fbcookie || !$fbcookie['uid'] || !$fbcookie['access_token'] ) {
                return;		// current user is not a facebook user or not logged in via Facebook
                                // or for some reason we do not have an access token.
                                // In all those cases ...just ignore and continue normally.
        }
        global $CURRENT_ALBUM_DATA;
        $aid = $CURRENT_ALBUM_DATA['aid'];
        if(!is_numeric($aid)) {
                return;		// non-numeric album info could be "topn" or similar -- we will not not deal with those.
        }
        $result = cpg_db_query("SELECT visibility FROM {$CONFIG['TABLE_ALBUMS']} WHERE aid = {$aid}");
        if(mysql_num_rows($result) > 0) {
                $row = mysql_fetch_array($result);
                $visibility = $row['visibility'];
                mysql_free_result($result);
                if($visibility != 0) {	// only check if album is non-public
                        print '<img src="images/spacer.gif" width="1" height="7" border="" alt="" />';
                        starttable('100%', '', 1, 'cpg_zebra');
                        $result = cpg_db_query("SELECT fb_uid FROM {$CONFIG['TABLE_USERS']} AS u INNER JOIN {$CONFIG['TABLE_PREFIX']}plugin_fb_oauth_usermap AS m ON m.cpg_user_id = u.user_id WHERE u.user_group = {$visibility} OR u.user_group_list = '{$visibility}' OR u.user_group_list LIKE '{$visibility},%' OR u.user_group_list LIKE '%,{$visibility},%' OR u.user_group_list LIKE '%,{$visibility}'");
                        if(mysql_num_rows($result) > 0) {
                                // Facebook API call
                                $friends = json_decode(file_get_contents('https://graph.facebook.com/me/friends?access_token='.$fbcookie['access_token']));
                                $friends = $friends->data;	// The result is packaged in "data"
                                $alloweduids = array();
                                do {
                                        $row = mysql_fetch_array($result);
                                        if($row) {
                                                $alloweduids[] = $row['fb_uid'];
                                        }
                                } while($row);
                                mysql_free_result($results);
                                
                                // Wanna keep the friend array order as provided from Facebook (Is that useful?)
                                $numFriends = count($friends);
                                for($i=0; $i<$numFriends; $i++) { // a boobytrap -- we are modifying the array in the loop but still want to go over all elements. So we have to use a pre-assigned count.
                                        if(!in_array ($friends[$i]->id,$alloweduids)) {
                                              unset($friends[$i]);	// PHP can delete the array element without reindexing.
                                        }
                                }
                                if (empty($friends)) {
                                        print '<tr><td class="tableh1">'.cpg_fetch_icon('info', 0).$lang_plugin_fb_oauth['noonecanseealbum'].'</td></tr>';
                                }
                                else {
                                        print '<tr><td class="tableh1">'.cpg_fetch_icon('info', 0).$lang_plugin_fb_oauth['whocanseealbum'].'</td></tr>';
                                        print '<tr><td class="thumbnails">';
                                        foreach ($friends as $friend) {
/*	// Reading profile link information takes long and is not parallelized.
        // do we really need it? Probably not, so I'll leave this as commented.
                                                $info = json_decode(file_get_contents('https://graph.facebook.com/'.$friend->id.'?access_token='.$fbcookie['access_token']));
                                                print '<a href="'.$info->link.'">';
*/
                                                print '<img src="https://graph.facebook.com/'.$friend->id.'/picture" border="0" alt="'.htmlspecialchars($friend->name).'" title="'.htmlspecialchars($friend->name).'" />';
/*                                                print '</a>';
*/
                                        }
                                        print '</td></tr>';
                                }
                        } else {
                                print '<tr><td class="tableh1">'.cpg_fetch_icon('info', 0).$lang_plugin_fb_oauth['noonecanseealbum'].'</td></tr>';
                        }
                        endtable();
                }
        }
}

function fb_oauth_get_facebook_cookie($app_id, $application_secret) {
  $args = array();
  $superCage = Inspekt::makeSuperCage();
  parse_str(trim($superCage->cookie->getRaw('fbs_' . $app_id), '\\"'), $args);
  ksort($args);
  $payload = '';
  foreach ($args as $key => $value) {
    if ($key != 'sig') {
      $payload .= $key . '=' . $value;
    }
  }
  if (md5($payload . $application_secret) != $args['sig']) {
    return null;
  }
  return $args;
}

?>
